{ ******************************************
  **** SPRITE LIBRARY - Frame list object
  ****
  **** Version 0.2
  **** Created by Chris Evans
  ******************************************* }

unit SFList;
interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, SFrame;

type
  ptrFrameNode = ^FrameNode;
  FrameNode = record
                iIndex:integer;
                ptrNext:ptrFrameNode;
                ptrPrev:ptrFrameNode;
                FFrame:TSFrame;
              end;

  TSFList = class(Tcomponent)
  private
    { Private declarations }
    ptrNext:ptrFrameNode;
    ptrCurrent:ptrFrameNode;
    ptrFirst:ptrFrameNode;
    ptrLast:ptrFrameNode;
    iFrameCount:integer;
    bDebugOn:boolean;
  protected
    { Protected declarations }
    procedure Debug(sz:string);
  public
    { Public declarations }
    constructor Create(Owner:TComponent); override;
    destructor Destroy; override;
    function ptrCreateFrame(szFilename, szDescription:string; iIndex:integer):ptrFrameNode;
    function ptrAddFrame(FFrame:TSFrame; iIndex:integer):ptrFrameNode;
    function ptrSetCurrentbyIndex(iIndex:integer):ptrFrameNode;
    function ptrSetCurrentbyPtr(ptrSet:ptrFrameNode):ptrFrameNode;
    function ptrGetCurrent:ptrFrameNode;
    function ptrGetFirst:ptrFrameNode;
    function ptrGetLast:ptrFrameNode;
    function ptrNextCurrent:ptrFrameNode;
    function ptrPrevCurrent:ptrFrameNode;
    procedure ListOrder;
  published
    { Published declarations }
    property DebugOn:boolean read bDebugOn write bDebugOn default true;
    property FrameCount:integer read iFrameCount default 0;
  end;

procedure Register;

implementation

procedure TSFList.Debug(sz:string);
begin
  {if bDebugOn then}
  begin
    sz:=concat('Sprite Frame List: ',sz,' (',TimeToStr(time),')',#10#13#0);
    OutputDebugString(@sz[1]);
  end;
end;

function TSFList.ptrNextCurrent:ptrFrameNode;
begin
  if ptrCurrent<>NIL then
    ptrCurrent:=ptrCurrent^.ptrNext;

  ptrNextCurrent:=ptrCurrent;
end;

function TSFList.ptrPrevCurrent:ptrFrameNode;
begin
  if ptrCurrent<>NIL then
    ptrCurrent:=ptrCurrent^.ptrPrev;

  ptrPrevCurrent:=ptrCurrent;
end;

function TSFList.ptrSetCurrentbyIndex(iIndex:integer):ptrFrameNode;
var ptrTraverse:ptrFrameNode;
begin
  debug('ptrSetCurrentbyIndex started');
  ptrTraverse:=ptrFirst;

  while (ptrTraverse<>NIL) and
        (iIndex<>ptrTraverse^.iIndex) do
  begin
    debug('Traverse!');
    ptrTraverse:=ptrTraverse^.ptrNext;
  end;

  ptrCurrent:=ptrTraverse;
  ptrSetCurrentbyIndex:=ptrCurrent;

  debug('ptrSetCurrentbyIndex ended');
end;

function TSFList.ptrSetCurrentbyPtr(ptrSet:ptrFrameNode):ptrFrameNode;
begin
  ptrCurrent:=ptrSet;
  ptrSetCurrentbyPtr:=ptrCurrent;
end;

function TSFList.ptrGetCurrent:ptrFrameNode;
begin
  ptrGetCurrent:=ptrCurrent;
end;

function TSFList.ptrGetFirst:ptrFrameNode;
begin
  ptrGetFirst:=ptrFirst;
end;

function TSFList.ptrGetLast:ptrFrameNode;
begin
  ptrGetLast:=ptrLast;
end;

procedure TSFList.ListOrder;
var ptrTraverse:ptrFrameNode;
    sz:string;
begin
  ptrTraverse:=ptrFirst;

  Debug('FORWARD');
  while ptrTraverse<>NIL do
  begin
    str(ptrTraverse^.iIndex,sz);
    Debug(sz);
    ptrTraverse:=ptrTraverse^.ptrNext;
  end;
  Debug('NIL');

  ptrTraverse:=ptrLast;
  Debug('BACKWARD');
  while ptrTraverse<>NIL do
  begin
    str(ptrTraverse^.iIndex,sz);
    Debug(sz);
    ptrTraverse:=ptrTraverse^.ptrPrev;
  end;
  Debug('NIL');
end;

function TSFList.ptrAddFrame(FFrame:TSFrame; iIndex:integer):ptrFrameNode;
var ptrNewFrameNode:ptrFrameNode;
    ptrTraverse:ptrFrameNode;
begin
  Debug('AddFrame started');

  New(ptrNewFrameNode);
  ptrNewFrameNode^.iIndex:=iIndex;
  ptrNewFrameNode^.FFrame:=FFrame;

  if ptrNext=NIL then
  begin
    {No nodes}
    Debug('No node addition');
    ptrNewFrameNode^.ptrNext:=NIL;
    ptrNewFrameNode^.ptrPrev:=NIL;

    ptrNext:=ptrNewFrameNode;
    ptrCurrent:=ptrNewFrameNode;
    ptrFirst:=ptrNewFrameNode;
    ptrLast:=ptrNewFrameNode;
  end
  else
  begin
    ptrTraverse:=ptrNext;
    while not (ptrTraverse^.ptrNext=NIL) and
          not (ptrTraverse^.ptrNext^.iIndex>iIndex) do
    begin
      Debug('Traverse!');
      ptrTraverse:=ptrTraverse^.ptrNext;
    end;

    if ptrTraverse^.ptrNext=NIL then
    begin
      if iIndex>ptrTraverse^.iIndex then
      begin
        {Last Node}
        Debug('Last node addition');
        ptrNewFrameNode^.ptrNext:=NIL;
        ptrNewFrameNode^.ptrPrev:=ptrTraverse;

        ptrTraverse^.ptrNext:=ptrNewFrameNode;

        ptrLast:=ptrNewFrameNode;
      end
      else
      begin
        {First Node}
        Debug('First node addition');
        ptrNewFrameNode^.ptrNext:=ptrTraverse;
        ptrNewFrameNode^.ptrPrev:=NIL;

        ptrTraverse^.ptrPrev:=ptrNewFrameNode;

        ptrNext:=ptrNewFrameNode;
        ptrFirst:=ptrNewFrameNode;
      end;
    end
    else
    begin
      if (ptrTraverse^.ptrPrev=NIL) and
         (iIndex<=ptrTraverse^.iIndex) then
      begin
        {First Node}
        Debug('First node addition');
        ptrNewFrameNode^.ptrNext:=ptrTraverse;
        ptrNewFrameNode^.ptrPrev:=NIL;

        ptrTraverse^.ptrPrev:=ptrNewFrameNode;

        ptrNext:=ptrNewFrameNode;
        ptrFirst:=ptrNewFrameNode;
      end
      else
      begin
        {Mid Node}
        Debug('Mid node addition');
        ptrNewFrameNode^.ptrNext:=ptrTraverse^.ptrNext;
        ptrNewFrameNode^.ptrPrev:=ptrTraverse;

        ptrTraverse^.ptrNext^.ptrPrev:=ptrNewFrameNode;
        ptrTraverse^.ptrNext:=ptrNewFrameNode;
      end;
    end;
  end;

  inc(iFrameCount);
  ptrAddFrame:=ptrNewFrameNode;
  Debug('AddFrame ended');
end;

function TSFList.ptrCreateFrame(szFilename, szDescription:string; iIndex:integer):ptrFrameNode;
var FNewFrame:TSFrame;
begin
  Debug('FCreateFrame started');
  FNewFrame:=TSFrame.Create(Self);

  FNewFrame.Description:=szDescription;
  FNewFrame.bLoadImage(szFilename);

  ptrCreateFrame:=ptrAddFrame(FNewFrame, iIndex);
  Debug('FCreateFrame ended');
end;

procedure Register;
begin
  RegisterComponents('Sprites', [TSFList]);
end;

constructor TSFList.Create(Owner:TComponent);
begin
  inherited Create(Owner);

  bDebugOn:=true;
  Debug('Create started');
  iFrameCount:=0;
  ptrNext:=NIL;
  ptrCurrent:=NIL;
  ptrFirst:=NIL;
  ptrLast:=NIL;
  Debug('Create ended');
end;

destructor TSFList.Destroy;
begin
  Debug('Destroy started');
  if ptrNext<>NIL then
      Dispose(ptrNext);

  Debug('Destroy ended');

  inherited Destroy;
end;

end.
