{ ******************************************
  **** SPRITE LIBRARY - Frame object
  ****
  **** Version 0.2
  **** Created by Chris Evans
  ******************************************* }

unit SFrame;

{$X+}

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs;

type
  TSFrame = class(TComponent)
  private
    { Private declarations }
    FAlpha:TPicture;
    FImage:TPicture;
    FTransColor:TColor;
    szDescription:string;
    bDebugOn:boolean;
  protected
    { Protected declarations }
    procedure Debug(sz:string);
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function bLoadImage(sz:string):boolean;
    procedure CreateMask;
  published
    { Published declarations }
    property Image:TPicture read FImage;
    property Alpha:TPicture read FAlpha;
    property Description:string read szDescription write szDescription;
    property DebugOn:boolean read bDebugOn write bDebugOn default true;
  end;

procedure Register;

implementation

procedure TSFrame.Debug(sz:string);
begin
  {if bDebugOn then}
  begin
    sz:=concat('Sprite Frame: ',sz,' (',TimeToStr(time),')',#10#13#0);
    OutputDebugString(@sz[1]);
  end;
end;

function TSFrame.bLoadImage(sz:string):boolean;
begin
  Debug('bLoadImage Started');
  FImage.LoadfromFile(sz);

  FTransColor:=FImage.Bitmap.Canvas.Pixels[0,0];
  CreateMask;

  bLoadImage:=true;
  Debug('bLoadImage Ended');
end;

Procedure TSFrame.CreateMask;
var lXLoop, lYLoop : longint;
begin
  Debug('CreateMask Started');

  FAlpha.Bitmap.Width:=FImage.Width;
  FAlpha.Bitmap.Height:=FImage.Height;

  For lXLoop:=0 to FImage.Width-1 do
    For lYLoop:=0 to FImage.Height-1 do
    begin
      If FImage.Bitmap.Canvas.Pixels[lXLoop, lYLoop]<>FTransColor then
      begin
        FAlpha.Bitmap.Canvas.Pixels[lXLoop, lYLoop]:=clBlack;
      end
      else
      begin
        FAlpha.Bitmap.Canvas.Pixels[lXLoop, lYLoop]:=clWhite;
        FImage.Bitmap.Canvas.Pixels[lXloop, lYLoop]:=clBlack;
      end;
    end;
    Debug('CreateMask End');
end;

constructor TSFrame.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  bDebugOn:=true;
  Debug('Create Started');
  FAlpha:=TPicture.Create;
  FImage:=TPicture.Create;
  Debug('Create Ended');
end;

destructor TSFrame.Destroy;
begin
  Debug('Destroy Started');
  FAlpha.Free;
  FImage.Free;

  inherited Destroy;
  Debug('Destroy Ended');
end;

procedure Register;
begin
  RegisterComponents('Sprites', [TSFrame]);
end;

end.
