{ ******************************************
  **** SPRITE LIBRARY - Sprite master object
  ****
  **** Version 0.2
  **** Created by Chris Evans
  ******************************************* }

unit SMaster;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, Sprite;

type
  ptrSpriteNode = ^SpriteNode;
  SpriteNode = record
                iIndex:integer;
                ptrNext:ptrSpriteNode;
                ptrPrev:ptrSpriteNode;
                FSprite:TSprite;
              end;

  TSMaster = class(TComponent)
  private
    { Private declarations }
    FDisplay:TImage;
    FActiveBackground:TPicture;
    FBackground:TPicture;
    FActiveRect:TRect;
    FDirtyRect:TRect;

    ptrNext:ptrSpriteNode;
    ptrFirst:ptrSpriteNode;
    ptrLast:ptrSpriteNode;

    iSpriteCount:integer;

    bActiveSet:boolean;
    bActiveChanged:boolean;
    bBackgroundLoaded:boolean;
    bDebugOn:boolean;
  protected
    { Protected declarations }
    procedure Debug(sz:string);
  public
    { Public declarations }
    constructor Create(Owner:TComponent); override;
    destructor Destroy; override;
    procedure SetDirtyRect(FRect:TRect);
    procedure ForceRefresh;
    procedure RefreshDisplay;
    procedure RefreshBuffer;
    procedure PaintBuffer;
    procedure SetActiveBackground(FRect:TRect);
    function FGetDisplay:TImage;
    procedure SetDisplay(Value:TImage);
    function FGetBackground:TPicture;
    procedure LoadBackground(szFilename:string);
    function FGetActiveBackground:TPicture;
    function ptrAddSprite(FSprite:TSprite; iIndex:integer):ptrSpriteNode;
    function ptrGetFirst:ptrSpriteNode;
    function ptrGetLast:ptrSpriteNode;
    procedure ListOrder;
  published
    { Published declarations }
    property SpriteCount:integer read iSpriteCount default 0;
  end;

procedure Register;

implementation

procedure TSMaster.Debug(sz:string);
begin
  {if bDebugOn then}
  begin
    sz:=concat('Sprite Master: ',sz,' (',TimeToStr(time),')',#10#13#0);
    OutputDebugString(@sz[1]);
  end;
end;

procedure TSMaster.SetDirtyRect(FRect:TRect);
begin
     FDirtyRect:=FRect;
end;

procedure TSMaster.ForceRefresh;
begin
     bActiveChanged:=True;
end;

function TSMaster.FGetDisplay:TImage;
begin
  FGetDisplay:=FDisplay;
end;

procedure TSMaster.SetDisplay(Value:TImage);
begin
  FDisplay:=Value;
end;

function TSMaster.FGetBackground:TPicture;
begin
  FGetBackground:=FBackground;
end;

function TSMaster.FGetActiveBackground:TPicture;
begin
  FGetActiveBackground:=FActiveBackground;
end;

procedure TSMaster.SetActiveBackground(FRect:TRect);
begin
  if bBackgroundLoaded then
    If (FRect.Left>=0) and (FRect.Top>=0) and
      (FRect.Right<=FBackground.Bitmap.Width) and (FRect.Bottom<=FBackground.Bitmap.Height) then
    begin
      bActiveSet:=True;
      bActiveChanged:=True;
      FActiveRect:=FRect;
    end;
end;

procedure TSMaster.LoadBackground(szFilename:string);
begin
  Debug('LoadBackground started');
  FBackground.LoadFromFile(szFilename);
  bBackgroundLoaded:=True;

  SetActiveBackground(Rect(0,0,FBackground.Width,FBackground.Height));
  Debug('LoadBackground ended');
end;

procedure TSMaster.RefreshDisplay;
var FDestRect, FSrcRect:TRect;
    lWidth, lHeight:longint;
begin
  if bActiveSet then
  begin
    If bActiveChanged then
    begin
       lWidth:=FActiveRect.Right-FActiveRect.Left;
       lHeight:=FActiveRect.Bottom-FActiveRect.Top;

       {If lWidth<>FDisplay.Width then
          FDisplay.Width := lWidth;

       If lHeight<>FDisplay.Height then
          FDisplay.Height := lHeight;}

       FSrcRect:=Rect(0,0,lWidth,lHeight);
       FDestRect:=Rect(0,0,lWidth,lHeight);

       FDisplay.Canvas.CopyMode := SRCCOPY;
       FDisplay.Picture.Bitmap.Canvas.CopyRect(FDestRect,FActiveBackground.Bitmap.Canvas,FSrcRect);

       bActiveChanged:=False;
    end
    else
    begin
       FDestRect:=FDirtyRect;

       FDisplay.Canvas.CopyMode:=SRCCOPY;
       FDisplay.Canvas.CopyRect(FDestRect,FActiveBackground.Bitmap.Canvas,FDirtyRect);
    end
  end;
end;

procedure TSMaster.RefreshBuffer;
var FDestRect, FSrcRect:TRect;
    lWidth, lHeight:longint;
begin
  if bActiveSet then
  begin
    If bActiveChanged then
    begin
       lWidth:=FActiveRect.Right-FActiveRect.Left;
       lHeight:=FActiveRect.Bottom-FActiveRect.Top;

       FActiveBackground.Bitmap.Width := lWidth;
       FActiveBackground.Bitmap.Height := lHeight;

       FDestRect:=Rect(0,0,lWidth,lHeight);

       FActiveBackground.Bitmap.Canvas.CopyMode:=SRCCOPY;
       FActiveBackground.Bitmap.Canvas.CopyRect(FDestRect,FBackground.Bitmap.Canvas,FActiveRect);
    end
    else
    begin
       lWidth:=FActiveRect.Right-FActiveRect.Left;
       lHeight:=FActiveRect.Bottom-FActiveRect.Top;


       FSrcRect:=Rect(FActiveRect.Left+FDirtyRect.Left, FActiveRect.Top+FDirtyRect.Top,
                      FActiveRect.Right+FDirtyRect.Right, FActiveRect.Bottom+FDirtyRect.Bottom);
       FDestRect:=FDirtyRect;

       FActiveBackground.Bitmap.Canvas.CopyMode:=SRCCOPY;
       FActiveBackground.Bitmap.Canvas.CopyRect(FDestRect,FBackground.Bitmap.Canvas,FSrcRect);
    end
  end;
end;

procedure TSMaster.PaintBuffer;
begin
end;

function TSMaster.ptrGetFirst:ptrSpriteNode;
begin
  ptrGetFirst:=ptrFirst;
end;

function TSMaster.ptrGetLast:ptrSpriteNode;
begin
  ptrGetLast:=ptrLast;
end;

procedure TSMaster.ListOrder;
var ptrTraverse:ptrSpriteNode;
    sz:string;
begin
  ptrTraverse:=ptrFirst;

  Debug('FORWARD');
  while ptrTraverse<>NIL do
  begin
    str(ptrTraverse^.iIndex,sz);
    Debug(sz);
    ptrTraverse:=ptrTraverse^.ptrNext;
  end;
  Debug('NIL');

  ptrTraverse:=ptrLast;
  Debug('BACKWARD');
  while ptrTraverse<>NIL do
  begin
    str(ptrTraverse^.iIndex,sz);
    Debug(sz);
    ptrTraverse:=ptrTraverse^.ptrPrev;
  end;
  Debug('NIL');
end;

function TSMaster.ptrAddSprite(FSprite:TSprite; iIndex:integer):ptrSpriteNode;
var ptrNewSpriteNode:ptrSpriteNode;
    ptrTraverse:ptrSpriteNode;
begin
  Debug('AddSprite started');

  {Use sprite master background now}
  FSprite.PrivateBackground := False;
  FSprite.Background:=FActiveBackground;

  New(ptrNewSpriteNode);
  ptrNewSpriteNode^.iIndex:=iIndex;
  ptrNewSpriteNode^.FSprite:=FSprite;

  if ptrNext=NIL then
  begin
    {No nodes}
    Debug('No node addition');
    ptrNewSpriteNode^.ptrNext:=NIL;
    ptrNewSpriteNode^.ptrPrev:=NIL;

    ptrNext:=ptrNewSpriteNode;
    ptrFirst:=ptrNewSpriteNode;
    ptrLast:=ptrNewSpriteNode;
  end
  else
  begin
    ptrTraverse:=ptrNext;
    while not (ptrTraverse^.ptrNext=NIL) and
          not (ptrTraverse^.ptrNext^.iIndex>iIndex) do
    begin
      Debug('Traverse!');
      ptrTraverse:=ptrTraverse^.ptrNext;
    end;

    if ptrTraverse^.ptrNext=NIL then
    begin
      if iIndex>ptrTraverse^.iIndex then
      begin
        {Last Node}
        Debug('Last node addition');
        ptrNewSpriteNode^.ptrNext:=NIL;
        ptrNewSpriteNode^.ptrPrev:=ptrTraverse;

        ptrTraverse^.ptrNext:=ptrNewSpriteNode;

        ptrLast:=ptrNewSpriteNode;
      end
      else
      begin
        {First Node}
        Debug('First node addition');
        ptrNewSpriteNode^.ptrNext:=ptrTraverse;
        ptrNewSpriteNode^.ptrPrev:=NIL;

        ptrTraverse^.ptrPrev:=ptrNewSpriteNode;

        ptrNext:=ptrNewSpriteNode;
        ptrFirst:=ptrNewSpriteNode;
      end;
    end
    else
    begin
      if (ptrTraverse^.ptrPrev=NIL) and
         (iIndex<=ptrTraverse^.iIndex) then
      begin
        {First Node}
        Debug('First node addition');
        ptrNewSpriteNode^.ptrNext:=ptrTraverse;
        ptrNewSpriteNode^.ptrPrev:=NIL;

        ptrTraverse^.ptrPrev:=ptrNewSpriteNode;

        ptrNext:=ptrNewSpriteNode;
        ptrFirst:=ptrNewSpriteNode;
      end
      else
      begin
        {Mid Node}
        Debug('Mid node addition');
        ptrNewSpriteNode^.ptrNext:=ptrTraverse^.ptrNext;
        ptrNewSpriteNode^.ptrPrev:=ptrTraverse;

        ptrTraverse^.ptrNext^.ptrPrev:=ptrNewSpriteNode;
        ptrTraverse^.ptrNext:=ptrNewSpriteNode;
      end;
    end;
  end;

  inc(iSpriteCount);
  ptrAddSprite:=ptrNewSpriteNode;
  Debug('AddSprite ended');
end;

procedure Register;
begin
  RegisterComponents('Sprites', [TSMaster]);
end;

constructor TSMaster.Create(Owner:TComponent);
begin
  inherited Create(Owner);

  bDebugOn:=true;
  Debug('Create started');
  FActiveBackground:=TPicture.Create;
  FBackground:=TPicture.Create;
  FDirtyRect:=Rect(0,0,0,0);

  iSpriteCount:=0;
  ptrNext:=NIL;
  ptrFirst:=NIL;
  ptrLast:=NIL;
  Debug('Create ended');
end;

destructor TSMaster.Destroy;
begin
  Debug('Destroy started');
  if ptrNext<>NIL then
      Dispose(ptrNext);

  FActiveBackground.Destroy;
  FBackground.Destroy;

  Debug('Destroy ended');

  inherited Destroy;
end;

end.
