{ ******************************************
  **** SPRITE LIBRARY - Sprite object
  ****
  **** Version 0.2
  **** Created by Chris Evans
  ******************************************* }

unit Sprite;

interface

uses
  SysUtils,
  WinTypes,
  WinProcs,
  Messages,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  ExtCtrls,
  SFList;

type
  TSprite = class(TComponent)
  private
    { Private declarations }
    FFrames:TSFList;
    FBackground:TPicture;
    FSave:TPicture;

    bPrivateBackground:boolean;
    bDrawn:boolean;
    bDebugOn:boolean;

    szDescription:string;

    lMidX, lMidY:longint;
    lSaveX, lSaveY:longint;
    lXVel, lYVel:longint;

    iZOrder:integer;
  protected
    { Protected declarations }
    procedure Debug(sz:string);
  public
    { Public declarations }
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    function FDrawSprite(lX, lY : longint):TRect;
    procedure MoveSprite(lX, lY : longint);
    function FGetFrames:TSFList;
  published
    { Published declarations }
    property SaveImage:TPicture read FSave;
    property Background:TPicture read FBackground write FBackground;
    property Description:string read szDescription write szDescription;
    property Drawn:boolean read bDrawn write bDrawn default false;
    property PrivateBackground:boolean read bPrivateBackground write bPrivateBackground default true;
    property xCurrent:longint read lSaveX;
    property yCurrent:longint read lSaveY;
    property MidX:longint read lMidX write lMidX;
    property MidY:longint read lMidY write lMidY;
    property xVelocity:longint read lXVel write lXVel;
    property yVelocity:longint read lYVel write lYVel;
    property ZOrder:integer read iZOrder write iZOrder default 50;
    property DebugOn:boolean read bDebugOn write bDebugOn default true;
  end;

procedure Register;

implementation

function TSprite.FGetFrames:TSFList;
begin
  FGetFrames:=FFrames;
end;

procedure Register;
begin
  RegisterComponents('Sprites', [TSprite]);
end;

procedure TSprite.Debug(sz:string);
begin
  {if bDebugOn then}
  begin
    sz:=concat('Sprite: ',sz,' (',TimeToStr(time),')',#10#13#0);
    OutputDebugString(@sz[1]);
  end;
end;

function TSprite.FDrawSprite(lX, lY : longint):TRect;
var SrcWindow, DestWindow:TRect;
    lWidth, lHeight:longint;
    ptrCurrent:ptrFrameNode;
    FShadow:TPicture;
    FImage:TPicture;
    FAlphaMask:TPicture;
begin
  {debug('DrawSprite started');}

  ptrCurrent:=FFrames.ptrGetCurrent;
  if ptrCurrent<>NIL then
  begin
    FImage:=ptrCurrent^.FFrame.Image;
    FAlphaMask:=ptrCurrent^.FFrame.Alpha;

    lWidth:=FImage.Width;
    lHeight:=FImage.Height;

    FShadow:=TPicture.Create;
    FShadow.Bitmap.Width:=lWidth;
    FShadow.Bitmap.Height:=lHeight;

    FSave.Bitmap.Width:=lWidth;
    FSave.Bitmap.Height:=lHeight;

    SrcWindow:=Rect(lX,lY,lX+lWidth,lY+lHeight);
    DestWindow:=Rect(0,0,lWidth,lHeight);

    {Draw New}
    FShadow.Bitmap.Canvas.CopyMode:=cmSrcCopy;
    FShadow.Bitmap.Canvas.CopyRect(DestWindow,FBackground.Bitmap.Canvas,SrcWindow);

    FShadow.Bitmap.Canvas.CopyMode:=cmSrcAnd;
    FShadow.Bitmap.Canvas.CopyRect(DestWindow,FAlphaMask.Bitmap.Canvas,DestWindow);

    FShadow.Bitmap.Canvas.CopyMode:=cmSrcPaint;
    FShadow.Bitmap.Canvas.CopyRect(DestWindow,FImage.Bitmap.Canvas,DestWindow);

    FBackground.Bitmap.Canvas.CopyMode:=cmSrcCopy;
    FBackground.Bitmap.Canvas.CopyRect(SrcWindow,FShadow.Bitmap.Canvas,DestWindow);

    FShadow.Free;

    FDrawSprite:=SrcWindow;
  end;

  {debug('DrawSprite ended');}
end;

procedure TSprite.MoveSprite(lX, lY : longint);
var SrcWindow, DestWindow:TRect;
    ptrCurrent:ptrFrameNode;
    FShadow:TPicture;
    FImage:TPicture;
    FAlphaMask:TPicture;
begin
  {debug('MoveSprite started');}

  ptrCurrent:=FFrames.ptrGetCurrent;
  if ptrCurrent<>NIL then
  begin
    FImage:=ptrCurrent^.FFrame.Image;
    FAlphaMask:=ptrCurrent^.FFrame.Alpha;

    FShadow:=TPicture.Create;
    FShadow.Bitmap.Width:=FImage.Width;
    FShadow.Bitmap.Height:=FImage.Height;

    If not bDrawn then
    begin
      FSave.Bitmap.Width:=FImage.Width;
      FSave.Bitmap.Height:=FImage.Height;

      {Save Old}
      SrcWindow:=Rect(lX,lY,lX+FImage.Width,lY+FImage.Height);
      DestWindow:=Rect(0,0,FImage.Width,FImage.Height);
      FSave.Bitmap.Canvas.CopyRect(DestWindow,FBackground.Bitmap.Canvas,SrcWindow);

      {Draw New}
      FShadow.Bitmap.Canvas.CopyMode:=cmSrcCopy;
      FShadow.Bitmap.Canvas.CopyRect(DestWindow,FBackground.Bitmap.Canvas,SrcWindow);

      FShadow.Bitmap.Canvas.CopyMode:=cmSrcAnd;
      FShadow.Bitmap.Canvas.CopyRect(DestWindow,FAlphaMask.Bitmap.Canvas,DestWindow);

      FShadow.Bitmap.Canvas.CopyMode:=cmSrcPaint;
      FShadow.Bitmap.Canvas.CopyRect(DestWindow,FImage.Bitmap.Canvas,DestWindow);

      FBackground.Bitmap.Canvas.CopyMode:=cmSrcCopy;
      FBackground.Bitmap.Canvas.CopyRect(SrcWindow,FShadow.Bitmap.Canvas,DestWindow);

      bDrawn:=True;
    end
    else
    begin
      {Restore Window}
      SrcWindow:=Rect(0,0,FImage.Width,FImage.Height);
      DestWindow:=Rect(lSaveX,lSaveY,lSaveX+FImage.Width,lSaveY+FImage.Height);
      FBackground.Bitmap.Canvas.CopyMode:=cmSrcCopy;
      FBackground.Bitmap.Canvas.CopyRect(DestWindow,FSave.Bitmap.Canvas,SrcWindow);

      {Save Old}
      SrcWindow:=Rect(lX,lY,lX+FImage.Width,lY+FImage.Height);
      DestWindow:=Rect(0,0,FImage.Width,FImage.Height);
      FSave.Bitmap.Canvas.CopyRect(DestWindow,FBackground.Bitmap.Canvas,SrcWindow);

      {Draw New}
      FShadow.Bitmap.Canvas.CopyMode:=cmSrcCopy;
      FShadow.Bitmap.Canvas.CopyRect(DestWindow,FBackground.Bitmap.Canvas,SrcWindow);

      FShadow.Bitmap.Canvas.CopyMode:=cmSrcAnd;
      FShadow.Bitmap.Canvas.CopyRect(DestWindow,FAlphaMask.Bitmap.Canvas,DestWindow);

      FShadow.Bitmap.Canvas.CopyMode:=cmSrcPaint;
      FShadow.Bitmap.Canvas.CopyRect(DestWindow,FImage.Bitmap.Canvas,DestWindow);

      FBackground.Bitmap.Canvas.CopyMode:=cmSrcCopy;
      FBackground.Bitmap.Canvas.CopyRect(SrcWindow,FShadow.Bitmap.Canvas,DestWindow);
    end;
    lSaveX:=lX;
    lSaveY:=lY;
    FShadow.Free;
  end;
  {debug('MoveSprite ended');}

end;

constructor TSprite.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  bDebugOn:=true;
  debug('Create started');

  FSave:=TPicture.Create;
  FFrames:=TSFList.Create(Self);

  bDrawn:=False;
  bPrivateBackground:=True;

  lSaveX:=0;
  lSaveY:=0;
  iZOrder:=50;

  debug('Create ended');
end;

destructor TSprite.Destroy;
begin
  debug('Destroy started');
  FSave.Free;
  FFrames.Free;

  inherited Destroy;
  debug('Destroy ended');
end;

end.
